"use strict";
/*
 * Copyright (c) 2021, knokbak and contributors.
 *
 * The Birb.JS Project: https://birb.js.org
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
let Sharding;
let DevTools;
const ClientError_1 = __importDefault(require("../errors/ClientError"));
const ClientWarning_1 = __importDefault(require("../errors/ClientWarning"));
const OptionError_1 = __importDefault(require("../errors/OptionError"));
const GuildBlock_1 = __importDefault(require("./blocks/GuildBlock"));
const UserBlock_1 = __importDefault(require("./blocks/UserBlock"));
const Intents_1 = __importDefault(require("./Intents"));
const Websocket_1 = __importDefault(require("./websocket/Websocket"));
try {
    Sharding = require('@birbjs/sharding');
}
catch (e) {
    Sharding = null;
}
try {
    DevTools = require('@birbjs/devtools');
}
catch (e) {
    DevTools = null;
}
class Client {
    constructor(options) {
        this.valid = true;
        this.devtools = null;
        this.token = null;
        this.ws = null;
        this.me = null;
        this.shard = null;
        this.options = {
            intents: null,
            debug: false,
        };
        this.guilds = null;
        this.users = null;
        this.events = {};
        if (options.debug) {
            if (DevTools) {
                this.devtools = new DevTools();
            }
            else {
                throw new ClientError_1.default('@birbjs/devtools is not installed; install it with `npm install @birbjs/devtools` to use the debug mode');
            }
        }
        options = options || {};
        if ('intents' in options) {
            if (!(options.intents instanceof Intents_1.default)) {
                throw new OptionError_1.default('intents must be an instance of Intents\nMore info: https://birb.js.org/explained/intents');
            }
            if (options.intents.isEmpty()) {
                throw new OptionError_1.default('intents must have at least one flag set\nMore info: https://birb.js.org/explained/intents');
            }
        }
        else {
            throw new OptionError_1.default('intents must be provided\nMore info: https://birb.js.org/explained/intents');
        }
        this.options = Object.assign(this.options, options);
        this.guilds = new GuildBlock_1.default(this);
        this.users = new UserBlock_1.default(this, {
            maxSize: 150000,
            removeOldest: true,
        });
        if (Sharding && process.env.BIRB_IS_SHARD == 'true') {
            this.warn('sharding is enabled');
            this.debug(`shard id: ${process.env.BIRB_SHARD_NUMBER}`);
            this.debug(`total shards: ${process.env.BIRB_TOTAL_SHARDS}`);
            this.shard = new Sharding.Shard(this);
        }
    }
    /**
     * Add an event listener.
     *
     * @param {EventResolvable | string} event The event name.
     * @param {Function} listener The function to call when the event is emitted.
     * @returns {void}
     */
    listen(event, callback) {
        if (!this.valid) {
            throw new ClientError_1.default('the client has been invalidated; please restart the process');
        }
        if (this.events[event] !== undefined) {
            process.emitWarning(new ClientWarning_1.default('binding to an event multiple times is not supported: previous listener overwritten'));
        }
        this.events[event] = callback;
    }
    /**
     * Undind an event listener.
     *
     * @param {EventResolvable | string} event The event name to unbind.
     * @returns {void}
     */
    unbind(event) {
        if (!this.valid)
            throw new ClientError_1.default('the client has been invalidated; please restart the process');
        delete this.events[event];
    }
    /**
     * Emit an event.
     *
     * @param {EventResolvable | string} event The event name.
     * @param {any} data The event data to pass to the listener.
     * @returns {void}
     */
    emit(event, ...args) {
        if (this.events[event] == undefined || !this.valid)
            return;
        return this.events[event](...args);
    }
    /**
     * Initilize a new addon to the client.
     *
     * @param {Function} fn The addon's main function.
     * @returns {any} The updated client.
     */
    add(fn) {
        fn.bind(this)();
        return this;
    }
    /**
     * Connect to the Discord gateway.
     *
     * ⚠️ **DON'T SHARE YOUR GOD DAMN TOKEN!** There is no
     * reason anyone should need it, with the exception of
     * your development team. ANYONE with your token can
     * gain FULL and UNRESTRICTED ACCESS to the entirety of
     * your bot, INCLUDING ANY PERMISSIONS IT HAS BEEN
     * GRANTED. Bot takeovers have happened far too often
     * in the past: don't let your bot join the list.
     *
     * @param {string} token Your bot's token.
     * @returns {void}
     */
    connect(token) {
        if (!this.valid)
            throw new ClientError_1.default('the client has been invalidated; please restart the process');
        if (this.devtools)
            this.devtools.setToken(token);
        this.token = token;
        Object.freeze(this.token);
        Object.freeze(this.options);
        this.ws = new Websocket_1.default(this, 'gateway.discord.gg');
        this.ws.connect();
    }
    /**
     * Emit a log in debug mode.
     *
     * @param {any} message The log message.
     * @returns {void}
     */
    debug(...message) {
        if (this.options.debug && this.devtools) {
            this.devtools.logger.log(...message);
        }
    }
    /**
     * Emit a warning in debug mode.
     *
     * @param {any} message The warning message.
     * @returns {void}
     */
    warn(...message) {
        if (this.options.debug && this.devtools) {
            this.devtools.logger.warn(...message);
        }
    }
    /**
     * Log a packet received from the gateway.
     *
     * @param {any} message The packet data.
     * @returns {void}
     */
    logReceive(...message) {
        if (this.options.debug && this.devtools) {
            this.devtools.logger.receive(...message);
        }
    }
    /**
     * Log a packet sent to the gateway.
     *
     * @param {any} message The packet data.
     * @returns {void}
     */
    logSend(...message) {
        if (this.options.debug && this.devtools) {
            this.devtools.logger.send(...message);
        }
    }
    /**
     * Log a HTTP event.
     *
     * @param {any} message The event data.
     * @returns {void}
     */
    logHttp(...message) {
        if (this.options.debug && this.devtools) {
            this.devtools.logger.http(...message);
        }
    }
    /**
     * Only registers the provided event if it isn't
     * already registered.
     */
    listenIfNotRegistered(event, callback) {
        if (this.events[event] == undefined)
            this.listen(event, callback);
    }
    /**
     * Invalidate the client, forcing the user to restart
     * the process to use it. Once called, this cannot be
     * undone (mostly because it would defeat the entire
     * purpose of this function).
     */
    invalidate() {
        this.events = {};
        this.valid = false;
        Object.freeze(this.valid);
        console.error(new ClientError_1.default('the Birb.JS client has been invalidated; please restart the process'));
    }
}
exports.default = Client;
