"use strict";
/*
 * Copyright (c) 2021, knokbak and contributors.
 *
 * The Birb.JS Project: https://birb.js.org
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const HTTPChannel_1 = __importDefault(require("./http/HTTPChannel"));
const Role_1 = __importDefault(require("./Role"));
const Embed_1 = __importDefault(require("./message/embed/Embed"));
const Constants_1 = require("../util/Constants");
const User_1 = __importDefault(require("./User"));
class Message {
    constructor(client, data) {
        this.client = null;
        this.content = null;
        this.webhookId = null;
        this.guild = null;
        this.author = null;
        this.member = null;
        this.channel = null;
        this.flags = 0;
        this.baseAuthor = null;
        this.client = client;
        this.id = data.id;
        this.build(data);
    }
    build(data) {
        this.baseAuthor = data.author || null;
        if ('content' in data) {
            this.content = data.content;
        }
        if ('webhook_id' in data) {
            this.webhookId = data.webhook_id;
        }
        if ('guild_id' in data) {
            this.guild = this.client.guilds.cache.get(data.guild_id) || null;
        }
        if ('member' in data) {
            try {
                data.member.user = this.baseAuthor;
                data.member.id = this.baseAuthor.id;
                this.member = this.guild?.members.resolve(this.baseAuthor.id, this.guild, data.member) || null;
            }
            catch (err) { }
        }
        if (!this.webhookId && !data.author.system) {
            if (data.author.id == this.client.me?.id) {
                this.author = this.client.me;
            }
            else {
                this.author = this.client.users.resolve(data.author.id, data.author ?? null) || null;
            }
        }
        if (this.guild) {
            this.channel = this.guild.channels.cache.get(data.channel_id);
        }
    }
    async crosspost() {
        return this.parse(await HTTPChannel_1.default.crossPostMessage(this.client, this.channel.id, this.id));
    }
    async delete() {
        await HTTPChannel_1.default.deleteMessage(this.client, this.channel.id, this.id);
    }
    async reply(message) {
        await this.waitForAuthor();
        let data = Message.buildApiMessage(this.client, message);
        data.message_reference = {
            message_id: this.id,
            channel_id: this.channel.id,
            fail_if_not_exists: false,
        };
        let msg = await HTTPChannel_1.default.createMessage(this.client, this.channel.id, data);
        return new Message(this.client, msg).waitForAuthor();
    }
    async pin(reason) {
        await HTTPChannel_1.default.pinMessage(this.client, this.channel.id, this.id, reason);
        return this;
    }
    async unpin(reason) {
        await HTTPChannel_1.default.unpinMessage(this.client, this.channel.id, this.id, reason);
        return this;
    }
    async startThread(name, options = {}) {
        await HTTPChannel_1.default.startThreadWithMessage(this.client, this.channel.id, this.id, {
            name: name,
            auto_archive_duration: options.autoArchiveDelay ?? null,
            rate_limit_per_user: options.slowmode ?? 0,
        }, options.reason);
    }
    async supressEmbeds() {
        return this.modify({
            flags: this.flags | Constants_1.MessageFlags.SUPPRESS_EMBEDS,
        });
    }
    async edit(message) {
        return this.modify(Message.buildApiMessage(this.client, message));
    }
    async modify(data) {
        let res = await HTTPChannel_1.default.editMessage(this.client, this.channel.id, this.id, data);
        this.build(res);
        return this.set();
    }
    async parse(data) {
        this.build(data);
        await this.waitForAuthor();
        return this.set();
    }
    async waitForAuthor() {
        if (!this.author && !this.webhookId && !this.baseAuthor.system) {
            if (this.client.me && this.baseAuthor.id === this.client.me.id) {
                this.author = this.client.me;
            }
            else {
                this.author = await this.client.users.fetch(this.baseAuthor.id);
            }
        }
        return this;
    }
    set() {
        if (this.guild) {
            this.guild.channels.cache.get(this.channel.id).messages.cache.set(this.id, this);
        }
        return this;
    }
    static buildApiMessage(client, data) {
        if (typeof data === 'string') {
            return { content: data };
        }
        if (data instanceof Embed_1.default) {
            return { embeds: [data.format()] };
        }
        return {
            content: data.content ?? null,
            embeds: data.embeds?.map(e => e.format()) ?? [],
            tts: data.tts ?? false,
            nonce: data.nonce ?? null,
            allowed_mentions: {
                parse: data.allowedMentions?.parse ?? null,
                users: data.allowedMentions?.users?.map((u) => User_1.default['toIdOnly'](u)) ?? null,
                roles: data.allowedMentions?.roles?.map((r) => Role_1.default['toIdOnly'](r)) ?? null,
                replied_user: data.mentionRepliedUser ?? false,
            },
        };
    }
}
exports.default = Message;
