import Client from './Client';
import Guild from './Guild';
import RolePermissionsBlock from './blocks/RolePermissionsBlock';
export default class Role {
    client: Client;
    readonly id: string;
    guild: Guild;
    name: string | null;
    color: string | null;
    hoist: boolean | null;
    icon: string | null;
    managed: boolean | null;
    mentionable: boolean | null;
    permissions: RolePermissionsBlock;
    position: number | null;
    unicodeEmoji: string | null;
    constructor(client: Client, data: any, guild: Guild);
    private build;
    /**
     * Delete this Role.
     *
     * @param {string} [reason] The reason for deleting this Role.
     * @returns {Promise<void>} A Promise that voids when the Role has been deleted.
     */
    delete(reason?: string): Promise<void>;
    /**
     * Set the name of this Role.
     *
     * @param {string} name The name to set this Role to.
     * @param {string} [reason] The reason for modifying this Role.
     * @returns {Promise<Role>} A Promise that resolves to the modified Role.
     */
    setName(name: string, reason?: string): Promise<Role>;
    /**
     * Set the position of this Role.
     *
     * @param {number} position The position to set this Role to.
     * @param {string} [reason] The reason for modifying this Role.
     * @returns {Promise<Role>} A Promise that resolves to the modified Role.
     */
    setPosition(position: number, reason?: string): Promise<Role>;
    /**
     * Move this Role's position.
     *
     * @param {number} velocity The velocity to apply to this Role's position. Provide a positive integer to move this Role up, and a negative integer to move this Role down.
     * @param {string} [reason] The reason for modifying this Role.
     * @returns {Promise<Role>} A Promise that resolves to the modified Role.
     */
    move(velocity: number, reason?: string): Promise<Role>;
    /**
     * Set the color of this Role.
     *
     * @param {number | string} color The new color for this Role. Accepts HEX color codes (e.g. ffffff for white) or an RGB integer.
     * @param {string} [reason] The reason for modifying this Role.
     * @returns {Promise<Role>} A Promise that resolves to the modified Role.
     */
    setColor(color: number | string, reason?: string): Promise<Role>;
    /**
     * Set if this Role can be mentioned.
     *
     * @param {boolean} mentionable Whether or not this Role can be mentioned.
     * @param {string} [reason] The reason for modifying this Role.
     * @returns {Promise<Role>} A Promise that resolves to the modified Role.
     */
    setIsMentionable(mentionable: boolean, reason?: string): Promise<Role>;
    /**
     * Set if this members of this Role should be hoisted
     * on the member list.
     *
     * @param {boolean} hoisted Whether or not this Role should be hoisted on the member list.
     * @param {string} [reason] The reason for modifying this Role.
     * @returns {Promise<Role>} A Promise that resolves to the modified Role.
     */
    setIsHoisted(hoisted: boolean, reason?: string): Promise<Role>;
    /**
     * Modify this Role.
     *
     * @param {Object} data The data to modify this Role with.
     * @param {string} [reason] The reason for modifying this Role.
     * @returns {Promise<Role>} A Promise that resolves to the modified Role.
     */
    modify(data: any, reason?: string): Promise<Role>;
    /**
     * Convert this Role into a mention (string).
     *
     * @returns {string} A string that mentions this role.
     */
    toString(): string;
    /**
     * Handle updates from the Discord Gateway.
     *
     * @param {Client} client The client that received the update.
     * @param {any} data The data to handle.
     * @param {Guild} guild The Guild this Role belongs to.
     * @returns {Role} A reference to this Role.
     */
    private static handleUpdate;
    /**
     * Set the Role's data to the cache.
     *
     * @returns {Role} This Role instance.
     * @private
     */
    private set;
    private static toIdOnly;
}
