import BaseComponent from '../BaseComponent';
import EmbedAuthor from './EmbedAuthor';
import EmbedField from './EmbedField';
import EmbedFooter from './EmbedFooter';
import EmbedMedia from './EmbedMedia';
export default class Embed extends BaseComponent {
    title: string | null;
    description: string | null;
    url: string | null;
    timestamp: Date | null;
    color: string | null;
    footer: EmbedFooter | null;
    image: EmbedMedia | null;
    thumbnail: EmbedMedia | null;
    video: EmbedMedia | null;
    author: EmbedAuthor | null;
    fields: EmbedField[];
    constructor();
    /**
     * Set the title of this embed.
     *
     * @param {string} title The new title of this embed.
     * @returns {Embed} This embed.
     */
    setTitle(title: string): Embed;
    /**
     * Set the description of this embed.
     *
     * @param {string} description The new description of this embed.
     * @returns {Embed} This embed.
     */
    setDescription(description: string): Embed;
    /**
     * Set the URL of this embed.
     *
     * @param {string} url The new URL of this embed.
     * @returns {Embed} This embed.
     */
    setUrl(url: string): Embed;
    /**
     * Set the timestamp of this embed.
     *
     * @param {Date | number} timestamp The new timestamp of this embed.
     * @returns {Embed} This embed.
     */
    setTimestamp(timestamp: Date | number): Embed;
    /**
     * Set the HEX color of this embed.
     *
     * @param {string} color The new HEX color of this embed.
     * @returns {Embed} This embed.
     */
    setColor(color: string): Embed;
    /**
     * Set the footer of this embed.
     *
     * @param {string} text The text of the footer.
     * @param {string} [iconUrl] A public-facing URL to an image for the footer.
     * @returns {Embed} This embed.
     */
    setFooter(text: string, iconUrl?: string): Embed;
    /**
     * Set the image for this embed.
     *
     * @param {string} url A public-facing URL to an image for the embed.
     * @param {number} [height] An optional height for the image.
     * @param {number} [width] An optional width for the image.
     * @returns {Embed} This embed.
     */
    setImage(url: string, height?: number, width?: number): Embed;
    /**
     * Set the thumbnail for this embed.
     *
     * @param {string} url A public-facing URL to an image for the embed.
     * @param {number} [height] An optional height for the image.
     * @param {number} [width] An optional width for the image.
     * @returns {Embed} This embed.
     */
    setThumbnail(url: string, height?: number, width?: number): Embed;
    /**
     * Set the video for this embed.
     *
     * @param {string} url A public-facing URL to a video for the embed.
     * @param {number} [height] An optional height for the video.
     * @param {number} [width] An optional width for the video.
     * @returns {Embed} This embed.
     */
    setVideo(url: string, height?: number, width?: number): Embed;
    /**
     * Set the author of this embed.
     *
     * @param {string} name The name of the author.
     * @param {string} [url] A public-facing URL for the author.
     * @param {string} [iconUrl] A public-facing URL to an image for the author.
     * @returns {Embed} This embed.
     */
    setAuthor(name: string, url?: string, iconUrl?: string): Embed;
    /**
     * Add a field to this embed.
     *
     * @param {string} name The name of the field.
     * @param {string} value The field's value.
     * @param {boolean} [inline=false] Whether or not this field should be displayed inline.
     * @returns {Embed} This embed.
     */
    addField(name: string, value: string, inline?: boolean): Embed;
    /**
     * Add multiple fields to this embed.
     *
     * @param {Object[]} fields The fields to add to this embed.
     * @param {string} fields.name The name of the field.
     * @param {string} fields.value The field's value.
     * @param {boolean} [fields.inline=false] Whether or not the field should be displayed inline.
     * @returns {Embed} This embed.
     */
    addFields(fields: {
        name: string;
        value: string;
        inline?: boolean;
    }[]): Embed;
    /**
     * Remove a field from this embed.
     *
     * @param {number} index The array index of the field to remove.
     * @returns {Embed} This embed.
     */
    removeField(index: number): Embed;
    /**
     * Set the fields of this embed.
     *
     * @param {Object[]} fields The new fields of this embed.
     * @param {string} fields.name The name of the field.
     * @param {string} fields.value The field's value.
     * @param {boolean} [fields.inline=false] Whether or not the field should be displayed inline.
     * @returns {Embed} This embed.
     */
    setFields(fields: {
        name: string;
        value: string;
        inline?: boolean;
    }[]): Embed;
    /**
     * Format this class into an API-acceptable object.
     */
    format(): {
        title: string | null;
        type: string;
        description: string | null;
        url: string | null;
        timestamp: string | undefined;
        color: number | null;
        footer: {
            text: string;
            icon_url: string | null;
        } | undefined;
        image: {
            url: string;
            height: number | null;
            width: number | null;
        } | undefined;
        thumbnail: {
            url: string;
            height: number | null;
            width: number | null;
        } | undefined;
        video: {
            url: string;
            height: number | null;
            width: number | null;
        } | undefined;
        author: {
            name: string;
            url: string | null;
            icon_url: string | null;
        } | undefined;
        fields: {
            name: string;
            value: string;
            inline: boolean;
        }[];
    };
}
